Package: opGMMassessment
Type: Package
Title: Optimized Automated Gaussian Mixture Assessment
Version: 0.4.1
Authors@R: c(
    person(given = "Jorn",
           family = "Lotsch",
           role = c("aut", "cre"),
           email = "j.lotsch@em.uni-frankfurt.de",
           comment = c(ORCID = "0000-0002-5818-6958")),
    person(given = "Sebastian",
           family = "Malkusch",
           role = c("aut"),
           email = "",
           comment = c(ORCID = "0000-0001-6766-140X")))
Maintainer: Jorn Lotsch <j.lotsch@em.uni-frankfurt.de>
Description: Necessary functions for optimized automated evaluation of the number and parameters of Gaussian mixtures in one-dimensional data. Various methods are available for parameter estimation and for determining the number of modes in the mixture. A detailed description of the methods ca ben found in Lotsch, J., Malkusch, S. and A. Ultsch. (2022) <doi:10.1016/j.imu.2022.101113>.
URL: https://github.com/JornLotsch/opGMMassessment
Depends: R (>= 3.5.0)
License: GPL-3
Encoding: UTF-8
LazyData: true
Imports: AdaptGauss, DataVisualizations, DistributionOptimization,
        cluster, mixtools, grDevices, methods, foreach, stats, utils,
        rlang, ggplot2, parallel, caTools, dplyr, mclust, mixAK,
        multimode, NbClust, ClusterR, doParallel
NeedsCompilation: no
Packaged: 2026-04-16 17:59:34 UTC; joern
Author: Jorn Lotsch [aut, cre] (ORCID: <https://orcid.org/0000-0002-5818-6958>),
  Sebastian Malkusch [aut] (ORCID:
    <https://orcid.org/0000-0001-6766-140X>)
Repository: CRAN
Date/Publication: 2026-04-17 05:10:57 UTC
